@@ -86,7 +86,7 @@ that appear after the value in `after' with '...'"
   "Put a section for the current file into your annotation file"
   (interactive)
   (unless (buffer-file-name)
-    (error "This buffer has no associated file."))
+    (error "This buffer has no associated file"))
   (org-annotate-file-show-section))
 
 (defun org-annotate-file-show-section (&optional buffer)
