@@ -1,6 +1,6 @@
 ;;; org-docbook.el --- DocBook exporter for org-mode
 ;;
-;; Copyright (C) 2007-2011 Free Software Foundation, Inc.
+;; Copyright (C) 2007-2012  Free Software Foundation, Inc.
 ;;
 ;; Emacs Lisp Archive Entry
 ;; Filename: org-docbook.el
@@ -150,6 +150,7 @@ avoid same set of footnote IDs being used multiple times."
 (defcustom org-export-docbook-footnote-separator "<superscript>, </superscript>"
   "Text used to separate footnotes."
   :group 'org-export-docbook
+  :version "24.1"
   :type 'string)
 
 (defcustom org-export-docbook-emphasis-alist
@@ -162,7 +163,7 @@ avoid same set of footnote IDs being used multiple times."
   "A list of DocBook expressions to convert emphasis fontifiers.
 Each element of the list is a list of three elements.
 The first element is the character used as a marker for fontification.
-The second element is a formatting string to wrap fontified text with.
+The second element is a format string to wrap fontified text with.
 The third element decides whether to protect converted text from other
 conversions."
   :group 'org-export-docbook
@@ -195,6 +196,7 @@ This XSLT stylesheet is used by
 Object (FO) files.  You can use either `fo/docbook.xsl' that
 comes with DocBook, or any customization layer you may have."
   :group 'org-export-docbook
+  :version "24.1"
   :type 'string)
 
 (defcustom org-export-docbook-xslt-proc-command nil
@@ -293,7 +295,7 @@ then use this command to convert it."
   (interactive "r")
   (let (reg docbook buf)
     (save-window-excursion
-      (if (eq major-mode 'org-mode)
+      (if (derived-mode-p 'org-mode)
 	  (setq docbook (org-export-region-as-docbook
 			 beg end t 'string))
 	(setq reg (buffer-substring beg end)
@@ -627,7 +629,7 @@ publishing directory."
 	    (insert org-export-docbook-doctype))
 	(insert "<!-- Date: " date " -->\n")
 	(insert (format "<!-- DocBook XML file generated by Org-mode %s Emacs %s -->\n"
-			org-version emacs-major-version))
+			(org-version) emacs-major-version))
 	(insert org-export-docbook-article-header)
 	(insert (format
 		 "\n  <title>%s</title>
@@ -1016,11 +1018,11 @@ publishing directory."
 	   (t
 	    ;; This line either is list item or end a list.
 	    (when (when (get-text-property 0 'list-item line)
-	   	      (setq line (org-export-docbook-list-line
-			  line
-			  (get-text-property 0 'list-item line)
-			  (get-text-property 0 'list-struct line)
-			  (get-text-property 0 'list-prevs line)))))
+		    (setq line (org-export-docbook-list-line
+				line
+				(get-text-property 0 'list-item line)
+				(get-text-property 0 'list-struct line)
+				(get-text-property 0 'list-prevs line)))))
 
 	    ;; Empty lines start a new paragraph.  If hand-formatted lists
 	    ;; are not fully interpreted, lines starting with "-", "+", "*"
@@ -1064,7 +1066,7 @@ publishing directory."
 	(if (eq major-mode (default-value 'major-mode))
 	    (nxml-mode)))
 
-      ;; Remove empty paragraphs. Replace them with a newline.
+      ;; Remove empty paragraphs.  Replace them with a newline.
       (goto-char (point-min))
       (while (re-search-forward
 	      "[ \r\n\t]*\\(<para>\\)[ \r\n\t]*</para>[ \r\n\t]*" nil t)
@@ -1353,10 +1355,10 @@ that need to be preserved in later phase of DocBook exporting."
     (concat replaced line)))
 
 (defun org-export-docbook-list-line (line pos struct prevs)
-  "Insert list syntax in export buffer. Return LINE, maybe modified.
+  "Insert list syntax in export buffer.  Return LINE, maybe modified.
 
 POS is the item position or line position the line had before
-modifications to buffer. STRUCT is the list structure. PREVS is
+modifications to buffer.  STRUCT is the list structure.  PREVS is
 the alist of previous items."
   (let* ((get-type
 	  (function
