@@ -1,6 +1,6 @@
 ;;; org-info.el --- Support for links to Info nodes from within Org-Mode
 
-;; Copyright (C) 2004-2011 Free Software Foundation, Inc.
+;; Copyright (C) 2004-2012 Free Software Foundation, Inc.
 
 ;; Author: Carsten Dominik <carsten at orgmode dot org>
 ;; Keywords: outlines, hypermedia, calendar, wp
@@ -48,9 +48,9 @@
   "Store a link to an Info file and node."
   (when (eq major-mode 'Info-mode)
     (let (link desc)
-      (setq link (org-make-link "info:"
-				(file-name-nondirectory Info-current-file)
-				"#" Info-current-node))
+      (setq link (concat "info:"
+			 (file-name-nondirectory Info-current-file)
+			 "#" Info-current-node))
       (setq desc (concat (file-name-nondirectory Info-current-file)
 			 "#" Info-current-node))
       (org-store-link-props :type "info" :file Info-current-file
