@@ -1,4 +1,5 @@
-;; Copyright (C) 2009-2011  Free Software Foundation, Inc.
+;;; org-special-blocks.el --- handle Org special blocks
+;; Copyright (C) 2009-2012  Free Software Foundation, Inc.
 
 ;; Author: Chris Gray <chrismgray@gmail.com>
 
@@ -51,7 +52,7 @@ interpreted by other mechanisms.")
 (defun org-special-blocks-make-special-cookies ()
   "Adds special cookies when #+begin_foo and #+end_foo tokens are
 seen.  This is run after a few special cases are taken care of."
-  (when (or (eq org-export-current-backend 'html) 
+  (when (or (eq org-export-current-backend 'html)
 	    (eq org-export-current-backend 'latex))
     (goto-char (point-min))
     (while (re-search-forward "^[ \t]*#\\+\\(begin\\|end\\)_\\(.*\\)$" nil t)
@@ -79,17 +80,17 @@ seen.  This is run after a few special cases are taken care of."
 (add-hook 'org-export-latex-after-blockquotes-hook
 	  'org-special-blocks-convert-latex-special-cookies)
 
-(defvar line)
+(defvar org-line)
 (defun org-special-blocks-convert-html-special-cookies ()
   "Converts the special cookies into div blocks."
-  ;; Uses the dynamically-bound variable `line'.
-  (when (string-match "^ORG-\\(.*\\)-\\(START\\|END\\)$" line)
+  ;; Uses the dynamically-bound variable `org-line'.
+  (when (and org-line (string-match "^ORG-\\(.*\\)-\\(START\\|END\\)$" org-line))
     (message "%s" (match-string 1))
-    (when (equal (match-string 2 line) "START")
+    (when (equal (match-string 2 org-line) "START")
       (org-close-par-maybe)
-      (insert "\n<div class=\"" (match-string 1 line) "\">")
+      (insert "\n<div class=\"" (match-string 1 org-line) "\">")
       (org-open-par))
-    (when (equal (match-string 2 line) "END")
+    (when (equal (match-string 2 org-line) "END")
       (org-close-par-maybe)
       (insert "\n</div>")
       (org-open-par))
